// This file is part of PUMA.
// Copyright (C) 1999-2003  The PUMA developer team.
//                                                                
// This program is free software;  you can redistribute it and/or 
// modify it under the terms of the GNU General Public License as 
// published by the Free Software Foundation; either version 2 of 
// the License, or (at your option) any later version.            
//                                                                
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
// GNU General Public License for more details.                   
//                                                                
// You should have received a copy of the GNU General Public      
// License along with this program; if not, write to the Free     
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
// MA  02111-1307  USA                                            

#ifndef __win_c_tree_h__
#define __win_c_tree_h__

namespace Puma {


// Syntax tree node hierarchy:
class CTree;
class   CT_Statement;
class     CT_AsmBlock;


} // namespace Puma

#include "Puma/CTree.h"

namespace Puma {


class CT_AsmBlock : public CT_Statement {
  CTree *_key;
  CTree *_begin;
  CTree *_end;

public:
  CT_AsmBlock (CTree *k, CTree *b, CTree *e) :
    _key (k), _begin (b), _end (e) {}
  static const char *NodeId ();
  const char *NodeName () const { return NodeId (); }  
  virtual int Sons () const { return 3; }
  virtual CTree *Son (int n) const {
    switch (n) {
      case 0: return _key;
      case 1: return _begin;
      case 2: return _end;
      default: return (CTree*)0;
    }
  }
  virtual void ReplaceSon (CTree *old_son, CTree *new_son) {
    if (old_son == _key) _key = new_son;
    else if (old_son == _begin) _begin = new_son;
    else if (old_son == _end) _end = new_son;
  }
};


} // namespace Puma

#endif /* __win_c_tree_h__ */
