// This file is part of PUMA.
// Copyright (C) 1999-2003  The PUMA developer team.
//                                                                
// This program is free software;  you can redistribute it and/or 
// modify it under the terms of the GNU General Public License as 
// published by the Free Software Foundation; either version 2 of 
// the License, or (at your option) any later version.            
//                                                                
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
// GNU General Public License for more details.                   
//                                                                
// You should have received a copy of the GNU General Public      
// License along with this program; if not, write to the Free     
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
// MA  02111-1307  USA                                            

#ifndef __CTypeInfo_h__
#define __CTypeInfo_h__

#include "Puma/Printable.h"

namespace Puma {


class CClassDatabase;
class CRecord;
class CClassInfo;
class CUnionInfo;
class CEnumInfo;
class CFunctionInfo;
class CTypeList;
class CTypeQualified;
class CTypeFunction;
class CTypeBitField; 
class CTypePointer; 
class CTypeMemberPointer; 
class CTypeTemplateParam; 
class CTypeAddress; 
class CTypeArray; 
class CTypeVarArray;
class CTypeClass;
class CTypeUnion;
class CTypeEnum;
class CTypePrimitive;
class CTypeRecord;
class CT_ExprList;
class CObjectInfo;
class CScopeInfo;
//class ostream;

class CTypeInfo : public Printable {
  // needed for type printing
  enum PrintState {
    PRINT_STD = 0,  // standard
    PRINT_PRE = 1,  // printing a declarator prefix like '*' or '&'
    PRINT_ABS = 2,  // printing absolute names
    PRINT_TPL = 4,  // printing template argument list
    PRINT_TDN = 8   // print typedef names instead of defined type
  };

public:
  enum TypeId {
    /* DO NOT CHANGE THIS ORDER */
    TYPE_BOOL,
    TYPE_SIGNED_CHAR,
    TYPE_UNSIGNED_CHAR,
    TYPE_CHAR,
    TYPE_UNSIGNED_SHORT,
    TYPE_SHORT,
    TYPE_UNSIGNED_INT,
    TYPE_WCHAR_T,
    TYPE_INT,
    TYPE_UNSIGNED_LONG,
    TYPE_LONG,
    TYPE_UNSIGNED_LONG_LONG,
    TYPE_LONG_LONG,
    TYPE_FLOAT,
    TYPE_DOUBLE,
    TYPE_LONG_DOUBLE,

    TYPE_VOID,
    TYPE_UNDEFINED,
    TYPE_UNKNOWN_T,
    TYPE_ELLIPSIS,
    
    TYPE_CLASS,
    TYPE_UNION,
    TYPE_ENUM,
    TYPE_POINTER,
    TYPE_ADDRESS,
    TYPE_MEMBER_POINTER,
    TYPE_FUNCTION,
    TYPE_ARRAY,
    TYPE_VAR_ARRAY,
    TYPE_QUALIFIED,
    TYPE_BIT_FIELD,
    
    TYPE_TEMPLATE_PARAM,
    TYPE_EMPTY
  };
  
  // internal representation of size_t and ptrdiff_t types
  static CTypeInfo *CTYPE_SIZE_T;
  static CTypeInfo *CTYPE_PTRDIFF_T;

private:
  CTypeInfo   *_Base;
  TypeId       _Id;
  CObjectInfo *_TypedefInfo;

public:
  CTypeInfo (CTypeInfo *, TypeId);
  ~CTypeInfo ();

  bool operator ==(const CTypeInfo &) const;
  bool operator !=(const CTypeInfo &) const;
  
  void print (ostream& os) const;
  void TypeText (ostream &, const char * = (const char*)0,
                 bool abs = false, bool tdef = false) const;
  long int Dimension () const;
  
  long int Size () const;
  long int Align () const;

  TypeId Id () const;

  CTypeInfo *BaseType () const;
  void BaseType (CTypeInfo *);

  bool isTypedef () const;
  CObjectInfo *TypedefInfo () const;
  CTypeInfo *TypedefInfo (CObjectInfo *);

  bool isComplete (unsigned long = 0) const;
  bool isConst () const;
  bool isVolatile () const;
  bool isRestrict () const;
  bool isDependent (bool = true) const;
  
  CRecord *Record () const;
  CClassInfo *ClassInfo () const;
  CUnionInfo *UnionInfo () const;
  CEnumInfo *EnumInfo () const;
  CFunctionInfo *FunctionInfo () const;

  CTypeInfo *PtrBaseType () const;
  CTypeList *ArgTypes () const;
  CTypeInfo *VirtualType () const;
  CTypeInfo *UnqualType () const;
    
  CTypeFunction *PtrToFct () const;
  CTypeArray *PtrToArray () const;
  
  CTypeQualified *TypeQualified () const;
  CTypeFunction *TypeFunction () const;
  CTypeBitField *TypeBitField () const; 
  CTypePointer *TypePointer () const; 
  CTypeMemberPointer *TypeMemberPointer () const; 
  CTypeAddress *TypeAddress () const; 
  CTypeArray *TypeArray () const; 
  CTypeVarArray *TypeVarArray () const;
  CTypeClass *TypeClass () const;
  CTypeUnion *TypeUnion () const;
  CTypeRecord *TypeRecord () const;
  CTypeEnum *TypeEnum () const;
  CTypePrimitive *TypePrimitive () const;
  CTypeInfo *TypeEmpty () const;
  CTypeTemplateParam *TypeTemplateParam () const;

  bool isQualified () const;
  bool isPointer () const;
  bool isPointerOrArray () const;
  bool isAddress () const;
  bool isRecord () const;
  bool isClass () const;
  bool isUnion () const;
  bool isArray () const;
  bool isFixedArray () const;
  bool isVarArray () const;
  bool isArithmetic () const;
  bool isScalar () const;
  bool isAggregate () const;
  bool isInteger () const;
  bool isVoid () const;
  bool isEnum () const;
  bool isReal () const;
  bool isMemberPointer () const;
  bool isUndefined () const;
  bool isFunction () const;
  bool isBitField () const;
  bool isMethod () const;
  bool isStdFunction () const;
  bool isTemplate () const;
  bool isTemplateInstance () const;
  bool isTemplateParam () const;
  bool isTypeParam () const;
  bool isNonTypeParam () const;
  bool isObject (unsigned long = 0) const;

  // Primitive types.
  bool is_bool () const;
  bool is_char () const;
  bool is_wchar_t () const;
  bool is_short () const;
  bool is_int () const;
  bool is_long () const;
  bool is_long_long () const;
  bool is_signed_char () const;
  bool is_unsigned_char () const;
  bool is_unsigned_short () const;
  bool is_unsigned_int () const;
  bool is_unsigned_long () const;
  bool is_unsigned_long_long () const;
  bool is_float () const;
  bool is_double () const;
  bool is_long_double () const;
  bool is_void () const;
  bool is_undefined () const;
  bool is_unknown_t () const;
  bool is_ellipsis () const;
  
  bool is_signed () const;
  bool is_unsigned () const;

  // arithmetic conversion rank (C only)
  unsigned conv_rank () const;
  // compare arithmetic types (C only)
  bool operator >(const CTypeInfo &) const;
  bool operator >=(const CTypeInfo &) const;
  bool operator <=(const CTypeInfo &) const;
  bool operator <(const CTypeInfo &) const;
  
  // rank C++ arithmetic types
  unsigned rank () const;
  
public:
  // Duplicate the given type structure.
#ifndef __puma // PUMA is confused by static and non-static declaration
  static
#endif
  CTypeInfo *Duplicate (const CTypeInfo *);
  // Duplicate this.
  CTypeInfo *Duplicate () const;

  // Destroy the given type structure.
  static void Destroy (CTypeInfo *);

  void Mangled (ostream &) const;

private:
  void TypeText (char, ostream &, const char * = (const char*)0) const;
  void printName (char, ostream &, CObjectInfo *) const;
  void printScope (char, ostream &, CObjectInfo *) const;
};

extern CTypeInfo CTYPE_EMPTY;

} // namespace Puma


#include "Puma/CTypePrimitive.h"
#include "Puma/CTypeQualified.h"
#include "Puma/CTypeAddress.h"
#include "Puma/CTypeBitField.h"
#include "Puma/CTypeArray.h"
#include "Puma/CTypeVarArray.h"
#include "Puma/CTypePointer.h"
#include "Puma/CTypeMemberPointer.h"
#include "Puma/CTypeFunction.h"
#include "Puma/CTypeEnum.h"
#include "Puma/CTypeRecord.h"
#include "Puma/CTypeClass.h"
#include "Puma/CTypeUnion.h"
#include "Puma/CTypeTemplateParam.h"


namespace Puma {

inline CTypeInfo::CTypeInfo (CTypeInfo *info, CTypeInfo::TypeId id) :
  _Base (info ? info : this), 
  _Id (id),
  _TypedefInfo (0)
 {}
inline CTypeInfo::~CTypeInfo () 
 {}

inline CTypeInfo *CTypeInfo::Duplicate () const 
 { 
#ifndef __puma
return Duplicate (this);
#endif
 }

inline CTypeInfo::TypeId CTypeInfo::Id () const 
 { return _Id; }

inline bool CTypeInfo::operator !=(const CTypeInfo &type) const 
 { return ! (*this == type); }

inline void CTypeInfo::print (ostream& out) const 
 { TypeText ((char)PRINT_STD, out); }
inline void CTypeInfo::TypeText (ostream &out, const char *t, bool abs, bool tdef) const
 { char flags = (char)(abs ? PRINT_ABS : PRINT_STD);
   if (tdef) flags |= (char)PRINT_TDN; TypeText (flags, out, t); }

inline long int CTypeInfo::Dimension () const
 { return TypeArray () ? TypeArray ()->Dimension () : 
          TypeBitField () ? TypeBitField ()->Dimension () : -1; }
 
inline bool CTypeInfo::isTypedef () const
 { return (bool)_TypedefInfo; }
inline CObjectInfo *CTypeInfo::TypedefInfo () const
 { return _TypedefInfo; }

inline bool CTypeInfo::isConst () const
 { return TypeQualified () && TypeQualified ()->isConst (); }
inline bool CTypeInfo::isVolatile () const
 { return TypeQualified () && TypeQualified ()->isVolatile (); }
inline bool CTypeInfo::isRestrict () const
 { return TypeQualified () && TypeQualified ()->isRestrict (); }
  
inline CRecord *CTypeInfo::Record () const
 { return TypeRecord () ? TypeRecord ()->Record () :
          TypeFunction () ? TypeFunction ()->Record () :
          TypeMemberPointer () ? TypeMemberPointer ()->Record () : (CRecord*)0; }
inline CEnumInfo *CTypeInfo::EnumInfo () const
 { return TypeEnum () ? TypeEnum ()->EnumInfo () : (CEnumInfo*)0; }
inline CFunctionInfo *CTypeInfo::FunctionInfo () const
 { return TypeFunction () ? TypeFunction ()->FunctionInfo () : (CFunctionInfo*)0; }

inline CTypeInfo *CTypeInfo::BaseType () const
 { assert(_Base); return _Base; }
inline void CTypeInfo::BaseType (CTypeInfo *type)
 { _Base = type; }
inline CTypeInfo *CTypeInfo::PtrBaseType () const
 { return VirtualType ()->BaseType (); }

inline CTypeList *CTypeInfo::ArgTypes () const
 { return TypeFunction () ? TypeFunction ()->ArgTypes () :
          TypeQualified () ? TypeQualified ()->ArgTypes () : (CTypeList*)0; }

inline CTypePrimitive *CTypeInfo::TypePrimitive () const
 { return _Id < TYPE_CLASS ? (CTypePrimitive*)this : (CTypePrimitive*)0; }
inline CTypeQualified *CTypeInfo::TypeQualified () const
 { return _Id == TYPE_QUALIFIED ? (CTypeQualified*)this : (CTypeQualified*)0; }
inline CTypeFunction *CTypeInfo::TypeFunction () const
 { return _Id == TYPE_FUNCTION ? (CTypeFunction*)this : (CTypeFunction*)0; }
inline CTypeBitField *CTypeInfo::TypeBitField () const 
 { return _Id == TYPE_BIT_FIELD ? (CTypeBitField*)this : (CTypeBitField*)0; }
inline CTypePointer *CTypeInfo::TypePointer () const
 { return _Id == TYPE_POINTER || _Id == TYPE_MEMBER_POINTER ? (CTypePointer*)this : (CTypePointer*)0; }
inline CTypeMemberPointer* CTypeInfo::TypeMemberPointer () const
 { return _Id == TYPE_MEMBER_POINTER ? (CTypeMemberPointer*)this : (CTypeMemberPointer*)0; }
inline CTypeAddress *CTypeInfo::TypeAddress () const
 { return _Id == TYPE_ADDRESS ? (CTypeAddress*)this : (CTypeAddress*)0; }
inline CTypeArray *CTypeInfo::TypeArray () const
 { return _Id == TYPE_ARRAY || _Id == TYPE_VAR_ARRAY ? (CTypeArray*)this : (CTypeArray*)0; }
inline CTypeVarArray *CTypeInfo::TypeVarArray () const
 { return _Id == TYPE_VAR_ARRAY ? (CTypeVarArray*)this : (CTypeVarArray*)0; }
inline CTypeRecord *CTypeInfo::TypeRecord () const
 { return _Id == TYPE_CLASS || _Id == TYPE_UNION ? (CTypeRecord*)this : (CTypeRecord*)0; }
inline CTypeClass *CTypeInfo::TypeClass () const
 { return _Id == TYPE_CLASS ? (CTypeClass*)this : (CTypeClass*)0; }
inline CTypeUnion *CTypeInfo::TypeUnion () const
 { return _Id == TYPE_UNION ? (CTypeUnion*)this : (CTypeUnion*)0; }
inline CTypeEnum *CTypeInfo::TypeEnum () const
 { return _Id == TYPE_ENUM ? (CTypeEnum*)this : (CTypeEnum*)0; }
inline CTypeInfo *CTypeInfo::TypeEmpty () const
 { return _Id == TYPE_EMPTY ? (CTypeInfo*)this : (CTypeInfo*)0; }
inline CTypeTemplateParam *CTypeInfo::TypeTemplateParam () const
 { return _Id == TYPE_TEMPLATE_PARAM ? (CTypeTemplateParam*)this : (CTypeTemplateParam*)0; }

inline CTypeInfo *CTypeInfo::VirtualType () const 
 { return TypeQualified () || TypeBitField () || TypeAddress () ?
          BaseType ()->VirtualType () : (CTypeInfo*)this; }
inline CTypeInfo *CTypeInfo::UnqualType () const 
 { return TypeQualified () ? BaseType ()->UnqualType () : (CTypeInfo*)this; }

inline bool CTypeInfo::isQualified () const 
 { return TypeQualified (); }
inline bool CTypeInfo::isTemplateParam () const 
 { return VirtualType ()->TypeTemplateParam (); }
inline bool CTypeInfo::isPointer () const 
 { return VirtualType ()->TypePointer (); }
inline bool CTypeInfo::isMemberPointer () const
 { return VirtualType ()->TypeMemberPointer (); }
inline bool CTypeInfo::isClass () const
 { return VirtualType ()->TypeClass (); }
inline bool CTypeInfo::isUnion () const
 { return VirtualType ()->TypeUnion (); }
inline bool CTypeInfo::isArray () const
 { return VirtualType ()->TypeArray (); }
inline bool CTypeInfo::isFixedArray () const
 { return VirtualType ()->TypeArray () && 
          VirtualType ()->TypeArray ()->isFixed (); }
inline bool CTypeInfo::isVarArray () const
 { return VirtualType ()->TypeVarArray (); }
inline bool CTypeInfo::isEnum () const
 { return VirtualType ()->TypeEnum (); }
inline bool CTypeInfo::isFunction () const
 { return VirtualType ()->TypeFunction (); }
inline bool CTypeInfo::isBitField () const
 { return TypeBitField (); }
inline bool CTypeInfo::isUndefined () const
 { return VirtualType ()->is_undefined (); }
inline bool CTypeInfo::isInteger () const 
 { return VirtualType ()->_Id < TYPE_FLOAT || VirtualType ()->isEnum (); }
inline bool CTypeInfo::isReal () const 
 { CTypeInfo *base = VirtualType ();
   return base->is_float () || base->is_double () || base->is_long_double (); }
inline bool CTypeInfo::isMethod () const 
 { CTypeInfo *base = VirtualType ();
   return base->TypeFunction () && base->TypeFunction ()->Record (); }
inline bool CTypeInfo::isStdFunction () const
 { return VirtualType ()->TypeFunction () && ! isMethod (); }
inline bool CTypeInfo::isPointerOrArray () const 
 { CTypeInfo *base = VirtualType ();
   return base->TypePointer () || base->TypeArray (); }
inline bool CTypeInfo::isAddress () const 
 { return TypeAddress () || ((TypeQualified () || TypeBitField ()) && 
                             BaseType ()->isAddress ()); }
inline bool CTypeInfo::isRecord () const 
 { return VirtualType ()->TypeRecord (); }
inline bool CTypeInfo::isScalar () const 
 { return isArithmetic () || isPointer (); }
inline bool CTypeInfo::isAggregate () const 
 { return isArray () || isClass () || isUnion (); }
inline bool CTypeInfo::isVoid () const
 { return VirtualType ()->is_void (); }
inline bool CTypeInfo::isObject (unsigned long pos) const
 { return ! isFunction () && (pos ? isComplete (pos) : true); }
inline bool CTypeInfo::isTypeParam () const
 { return isTemplateParam () && 
          VirtualType ()->TypeTemplateParam ()->isType (); }
inline bool CTypeInfo::isNonTypeParam () const
 { return isTemplateParam () && 
          ! VirtualType ()->TypeTemplateParam ()->isType (); }

inline bool CTypeInfo::is_bool () const 
 { return _Id == TYPE_BOOL; }
inline bool CTypeInfo::is_char () const 
 { return _Id == TYPE_CHAR; }
inline bool CTypeInfo::is_wchar_t () const 
 { return _Id == TYPE_WCHAR_T; }
inline bool CTypeInfo::is_short () const 
 { return _Id == TYPE_SHORT; }
inline bool CTypeInfo::is_int () const 
 { return _Id == TYPE_INT; }
inline bool CTypeInfo::is_long () const 
 { return _Id == TYPE_LONG; }
inline bool CTypeInfo::is_long_long () const 
 { return _Id == TYPE_LONG_LONG; }
inline bool CTypeInfo::is_signed_char () const 
 { return _Id == TYPE_SIGNED_CHAR; }
inline bool CTypeInfo::is_unsigned_char () const 
 { return _Id == TYPE_UNSIGNED_CHAR; }
inline bool CTypeInfo::is_unsigned_short () const 
 { return _Id == TYPE_UNSIGNED_SHORT; }
inline bool CTypeInfo::is_unsigned_int () const 
 { return _Id == TYPE_UNSIGNED_INT; }
inline bool CTypeInfo::is_unsigned_long () const 
 { return _Id == TYPE_UNSIGNED_LONG; }
inline bool CTypeInfo::is_unsigned_long_long () const 
 { return _Id == TYPE_UNSIGNED_LONG_LONG; }
inline bool CTypeInfo::is_float () const 
 { return _Id == TYPE_FLOAT; }
inline bool CTypeInfo::is_double () const 
 { return _Id == TYPE_DOUBLE; }
inline bool CTypeInfo::is_long_double () const 
 { return _Id == TYPE_LONG_DOUBLE; }
inline bool CTypeInfo::is_void () const 
 { return _Id == TYPE_VOID; }
inline bool CTypeInfo::is_unknown_t () const 
 { return _Id == TYPE_UNKNOWN_T; }
inline bool CTypeInfo::is_undefined () const 
 { return _Id == TYPE_UNDEFINED; }
inline bool CTypeInfo::is_ellipsis () const 
 { return _Id == TYPE_ELLIPSIS; }

inline bool CTypeInfo::operator <=(const CTypeInfo &type) const
 { return ! (*this > type); }
inline bool CTypeInfo::operator >=(const CTypeInfo &type) const
 { return ! (*this < type); }


} // namespace Puma

#endif /* __CTypeInfo_h__ */
