// This file is part of PUMA.
// Copyright (C) 1999-2003  The PUMA developer team.
//                                                                
// This program is free software;  you can redistribute it and/or 
// modify it under the terms of the GNU General Public License as 
// published by the Free Software Foundation; either version 2 of 
// the License, or (at your option) any later version.            
//                                                                
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
// GNU General Public License for more details.                   
//                                                                
// You should have received a copy of the GNU General Public      
// License along with this program; if not, write to the Free     
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
// MA  02111-1307  USA                                            

#ifndef __TokenProvider__
#define __TokenProvider__

// A TokenProvider reads tokens from a token source and supports unlimited
// look ahead.

#include "Puma/Token.h"
#include "Puma/TokenSource.h"
#include "Puma/BCList.h"

namespace Puma {

class TokenProvider {
  
  typedef BCList<Token*, 8192> TokenStore;
  
  PumaTokenSource &_source;
  TokenStore _tokens;
  TokenStore::Iterator _read_pos;

public:
  typedef TokenStore::Iterator State;

  TokenProvider (PumaTokenSource &s) : _source (s) {
    _read_pos = _tokens.add (_source.next ());
  }

  // get the token source object
  PumaTokenSource &source () const { return _source; }
  
  // get the next token
  Token *next () {
    // go to the next read position
    ++_read_pos;
    // did we reach the end of the list?
    if (_read_pos == _tokens.end ()) {
      // add a new token to the token list
      Token *token = _source.next (); // get the next token from the source
      _read_pos = _tokens.add (token);
    }
    return *_read_pos;
  }
  
  // get the last token again
  Token *current () const { return *_read_pos; }
  
  // get and restore the read position
  State get_state () { return _read_pos; }
  void set_state (State restored_pos) { _read_pos = restored_pos; }
};

} // namespace Puma

#endif /* __TokenProvider__ */
